/** 
 *   Davis Weather client daemon
 *
 *   (c) Copyright 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

#define _GNU_SOURCE

#include <errno.h> 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/file.h>
#include <limits.h>

#include "socket_srv.h"
#include "socket_client.h"
#include "antenna_ib_pk.h"
#include "my_stdio.h"
#include "iar_bus.h"
#include "dwt.h"


void intHandler (int signo) 
{
	SYSLOG(LOG_ERR, "Bus: write error, SIGPIPE trapped\n");
	return;
}


static void usage(void)
{
	static const char *msg[] =
	{
	"DWT client: antenna position system.",
	"",
	"Version: ""VERSION",
	"http://www.iar.unlp.edu.ar/~fede/pub/posicionador",
	"",
	"(c) Copyright 2004, 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,",
	"Instituto Argentino de Radio Astronomia (IAR).",
	"",
	"Usage:",
	"dwtcd [-vln]",
	" -v\t\t - Verbose (incremental).",
	" -n, --nodaemon\t - No run as a daemon.",
	" -l, --nolog\t - No write events to syslog.",
	""
	};
	unsigned j;
	for (j = 0; j < sizeof(msg)/sizeof(msg[0]); j++)
		fprintf(stderr, "\t%s\n", msg[j]);


	return;
}


void close_log(void)
{
        SYSLOG(LOG_INFO, "Program terminated.");
}


#if 0
char dwt_packet[] = {
	0x4c, 0x4f, 0x4f, 0x14, 0x00, 0x2d, 0x06, 0x4f, 0x74, 0x73, 
	0x02, 0x2b, 0x4c, 0x02, 0x06, 0x0c, 0x9e, 0x00, 0xff, 0xff,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0xff, 0x52, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0x00, 0x00, 0xff, 0xff, 0x7f, 0x00, 0x00, 0xff, 0xff,
	0x00, 0x00, 0xca, 0x00, 0xe7, 0x05, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf3, 0x01, 0x03,
	0xc0, 0x21, 0x03, 0xd7, 0x06, 0x0a, 0x0d, 0x3c, 0x29
};
#endif

int make_pk_reg(struct antenna_packet *anp)
{
	ushort *wind, *wind_deg;

	if(anp->h.size != LOOP_LEN)
		return -1;

	wind = (ushort *) GET_FIELD("WIND_SPEED", anp->data, U_KPH);
	wind_deg = (ushort *) GET_FIELD("WIND_DEGREE", anp->data, 0);
	PVERB(PINFO, 
	      "Wind Speed: %d, Wind Degree: %d\n",
	      *wind, *wind_deg);
	anp->h.cmd = WCM_WIND;
	anp->h.size = sprintf(anp->data, "%3d %3d", *wind, *wind_deg);

	free(wind);
	free(wind_deg);
	
	return 0;
}


int dwtcd_main_loop(int *fd, int *fd_dwt)
{
	fd_set rfds;
	struct timeval tv;
	int retval;
	struct ib_packet pk;
	struct antenna_packet *anp;
	int done;
	int log = 0;

	anp = (struct antenna_packet *) &pk.data;
	done = 0;
	if(ibus_on( *fd_dwt ) < 0) {
		done = 1;
		close(*fd_dwt);
		*fd_dwt = -1;
	}
	while( !done ) {
		FD_ZERO(&rfds);
		FD_SET(*fd_dwt, &rfds);
		tv.tv_sec = 4;
		tv.tv_usec = 0;
		retval = select(*fd_dwt + 1, &rfds, NULL, NULL, &tv);
		if (retval) {
			if (log) {
				SYSLOG(LOG_INFO,"Weather station on-line.");
				log = 0;
			}
			if (FD_ISSET(*fd_dwt, &rfds)) {
				memset(&pk, 0, sizeof(struct ib_packet));
				if (ibus_read(*fd_dwt, &pk) > 0) {
					if (make_pk_reg(anp) != 0) {
						done = 1;
						close(*fd_dwt);
						*fd_dwt = -1;
					} else {
						if (ibus_send_data( *fd, anp, anp->h.size + sizeof(struct antenna_packet_head)) < 0) {
							done = 1;
							close(*fd);
							*fd = -1;
						}
					}
				} else {
					done = 1;
					close(*fd_dwt);
					*fd_dwt = -1;
				}
			}
		} else {
			if (!log){
				SYSLOG(LOG_ERR,"Weather station timeout.");
				log++;
			}
		}
	}

	return 0;
}



int main(int argc, char *argv[]) 
{
	int fd = -1;
	int fd_dwt = -1;
	int log;
	char *program_name;
#ifndef NODAEMON
        int pid;
        int fd_pid;
        char pid_file[_POSIX_PATH_MAX];
#endif
	char *cp;
	int op;
	static struct option  const  longopts[] =
	{
		{"help", 0, 0, 'h'},
		{"verbose", 0, 0, 'v'},
		{"nodaemon", 0, 0, 'n'},
		{"nolog", 0, 0, 'l'},
		{0,  0,  0,  0}
	};
	int daemon_flag = 1;
	
	syslog_flag = 1;
	/* who is my name? */
        if ((cp = strrchr(argv[0], '/')) != NULL) program_name  = cp + 1;
        else program_name  = argv[0];

	opterr =  0;
	while  ((op = getopt_long(argc, argv, "vhnl",longopts, 0)) != EOF)
		switch( op ){
		case 'v':
			verbose_flag = (verbose_flag << 1) | 1;
			break;
#ifndef NODAEMON
		case 'n':
			daemon_flag = 0;
			break;
#endif
		case 'l':
			syslog_flag = 0;
			break;
		case 'h':
			usage();
			//help();
			exit(-1); 
		default:
			usage();
			exit(-1);
		}
#       ifndef NODAEMON
	if(daemon_flag){
		/* Ok, a daemon */
                verbose_flag = 0;
                if ( (pid=fork()) != 0 ) {
                        sprintf(pid_file, "/var/run/%s.pid", program_name);
                        fd_pid = open(pid_file, O_WRONLY|O_CREAT|O_SYNC,
                                      S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
			if (fd_pid > 0) {
				fd_printf(fd_pid,"%d\n", pid);
				close(fd_pid);
			}else{
				perror("Can't open pid file");
				fprintf(stderr, "%s\n", pid_file);
			}
                        exit(0);
                }
        }
#       endif
	if(syslog_flag){
		openlog(program_name, LOG_PID, LOG_DAEMON );
		atexit(close_log);
		SYSLOG(LOG_INFO, "Starting %s.", program_name);
	}
#if 1
	if (signal(SIGPIPE, intHandler) == SIG_ERR) {
		perror("Signal Failed (SIGPIPE).");
		exit(1);
	}
#endif
	dwt_make_hash_table();
	while(1) {
		/* dwt: registry on bus */
		log = 1;
		if (fd < 0) {
			while ((fd = ANTENNA_CALL_SOCKET()) < 0) {
				if (log) 
					SYSLOG(LOG_WARNING, 
					"Waiting connection to encoders.");
				log = 0;
				sleep(1);
			}
			SYSLOG(LOG_INFO,"Connected to antenna bus");

		}
		
		/* dwt: cal wdt server*/
		log = 1;
		if (fd_dwt < 0) {
			while((fd_dwt = DWT_CALL_SOCKET()) < 0) {
				if (log) 
					SYSLOG(LOG_WARNING, 
					"Waiting connection to dwt server.");
				log = 0;
				sleep(1);
			}
			SYSLOG(LOG_INFO,"Connected to dwt server");
		}
		/*
		if ( ibus_set_addr(fd, _BUS_ADDR) != _BUS_ADDR) {
			SYSLOG(LOG_ERR,"Survey bus addr falled.");
			exit(1);
		}
		*/

		dwtcd_main_loop(&fd, &fd_dwt);
	}
	dwt_destroy_hash_table(); 
 
	return 0; /* never return */
}


